@@ -3,6 +3,7 @@
 #include "pch.h"
 #include "validate.h"
 
+#define CRYPTOPP_ENABLE_NAMESPACE_WEAK 1
 #include "smartptr.h"
 #include "crc.h"
 #include "adler32.h"
@@ -12,10 +13,7 @@
 #include "sha.h"
 #include "tiger.h"
 #include "ripemd.h"
-#include "haval.h"
 
-#include "xormac.h"
-#include "md5mac.h"
 #include "hmac.h"
 #include "ttmac.h"
 
@@ -40,7 +38,8 @@ struct HashTestTuple
 		: input((byte *)input), output((byte *)output), inputLen(inputLen), repeatTimes(repeatTimes) {}
 
 	const byte *input, *output;
-	unsigned int inputLen, repeatTimes;
+	size_t inputLen;
+	unsigned int repeatTimes;
 };
 
 bool HashModuleTest(HashTransformation &md, const HashTestTuple *testSet, unsigned int testSetSize)
@@ -122,7 +121,7 @@ bool ValidateMD2()
 		HashTestTuple("12345678901234567890123456789012345678901234567890123456789012345678901234567890", "\xd5\x97\x6f\x79\xd8\x3d\x3a\x0d\xc9\x80\x6c\x3c\x66\xf3\xef\xd8")
 	};
 
-	MD2 md2;
+	Weak::MD2 md2;
 
 	cout << "\nMD2 validation suite running...\n\n";
 	return HashModuleTest(md2, testSet, sizeof(testSet)/sizeof(testSet[0]));
@@ -141,7 +140,7 @@ bool ValidateMD4()
 		HashTestTuple("12345678901234567890123456789012345678901234567890123456789012345678901234567890", "\xe3\x3b\x4d\xdc\x9c\x38\xf2\x19\x9c\x3e\x7b\x16\x4f\xcc\x05\x36")
 	};
 
-	MD4 md4;
+	Weak::MD4 md4;
 
 	cout << "\nMD4 validation suite running...\n\n";
 	return HashModuleTest(md4, testSet, sizeof(testSet)/sizeof(testSet[0]));
@@ -160,7 +159,7 @@ bool ValidateMD5()
 		HashTestTuple("12345678901234567890123456789012345678901234567890123456789012345678901234567890", "\x57\xed\xf4\xa2\x2b\xe3\xc9\x55\xac\x49\xda\x2e\x21\x07\xb6\x7a")
 	};
 
-	MD5 md5;
+	Weak::MD5 md5;
 
 	cout << "\nMD5 validation suite running...\n\n";
 	return HashModuleTest(md5, testSet, sizeof(testSet)/sizeof(testSet[0]));
@@ -197,7 +196,6 @@ bool ValidateSHA2()
 
 	cout << "\nSHA-384 validation suite running...\n\n";
 
-#ifdef WORD64_AVAILABLE
 	HashTestTuple testSet384[] = 
 	{
 		HashTestTuple("abc", "\xcb\x00\x75\x3f\x45\xa3\x5e\x8b\xb5\xa0\x3d\x69\x9a\xc6\x50\x07\x27\x2c\x32\xab\x0e\xde\xd1\x63\x1a\x8b\x60\x5a\x43\xff\x5b\xed\x80\x86\x07\x2b\xa1\xe7\xcc\x23\x58\xba\xec\xa1\x34\xc8\x25\xa7"),
@@ -206,13 +204,9 @@ bool ValidateSHA2()
 
 	SHA384 sha384;
 	pass = HashModuleTest(sha384, testSet384, sizeof(testSet384)/sizeof(testSet384[0])) && pass;
-#else
-	cout << "word64 not available, skipping SHA-384 validation." << endl;
-#endif
 
 	cout << "\nSHA-512 validation suite running...\n\n";
 
-#ifdef WORD64_AVAILABLE
 	HashTestTuple testSet512[] = 
 	{
 		HashTestTuple("abc", "\xdd\xaf\x35\xa1\x93\x61\x7a\xba\xcc\x41\x73\x49\xae\x20\x41\x31\x12\xe6\xfa\x4e\x89\xa9\x7e\xa2\x0a\x9e\xee\xe6\x4b\x55\xd3\x9a\x21\x92\x99\x2a\x27\x4f\xc1\xa8\x36\xba\x3c\x23\xa3\xfe\xeb\xbd\x45\x4d\x44\x23\x64\x3c\xe8\x0e\x2a\x9a\xc9\x4f\xa5\x4c\xa4\x9f"),
@@ -221,9 +215,6 @@ bool ValidateSHA2()
 
 	SHA512 sha512;
 	pass = HashModuleTest(sha512, testSet512, sizeof(testSet512)/sizeof(testSet512[0])) && pass;
-#else
-	cout << "word64 not available, skipping SHA-512 validation." << endl;
-#endif
 
 	return pass;
 }
@@ -232,7 +223,6 @@ bool ValidateTiger()
 {
 	cout << "\nTiger validation suite running...\n\n";
 
-#ifdef WORD64_AVAILABLE
 	HashTestTuple testSet[] =
 	{
 		HashTestTuple("", "\x32\x93\xac\x63\x0c\x13\xf0\x24\x5f\x92\xbb\xb1\x76\x6e\x16\x16\x7a\x4e\x58\x49\x2d\xde\x73\xf3"),
@@ -249,10 +239,6 @@ bool ValidateTiger()
 	Tiger tiger;
 
 	return HashModuleTest(tiger, testSet, sizeof(testSet)/sizeof(testSet[0]));
-#else
-	cout << "word64 not available, skipping Tiger validation." << endl;
-	return true;
-#endif
 }
 
 bool ValidateRIPEMD()
@@ -330,6 +316,7 @@ bool ValidateRIPEMD()
 	return pass;
 }
 
+#ifdef CRYPTOPP_REMOVED
 bool ValidateHAVAL()
 {
 	HashTestTuple testSet[] = 
@@ -372,6 +359,7 @@ bool ValidateHAVAL()
 
 	return pass;
 }
+#endif
 
 bool ValidatePanama()
 {
@@ -384,14 +372,10 @@ bool ValidateWhirlpool()
 {
 	cout << "\nWhirlpool Hash Function validation suite running...\n";
 
-#ifdef WORD64_AVAILABLE
 	return RunTestDataFile("TestVectors/whrlpool.txt");
-#else
-	cout << "word64 not available, skipping Whirlpool validation." << endl;
-	return true;
-#endif
-	}
+}
 
+#ifdef CRYPTOPP_REMOVED
 bool ValidateMD5MAC()
 {
 	const byte keys[2][MD5MAC::KEYLENGTH]={
@@ -451,10 +435,11 @@ bool ValidateMD5MAC()
 
 	return pass;
 }
+#endif
 
 bool ValidateHMAC()
 {
-	typedef HMAC<MD5> HMAC_MD5;
+	typedef HMAC<Weak::MD5> HMAC_MD5;
 
 	const char* keys[]=
 	{
@@ -495,6 +480,7 @@ bool ValidateHMAC()
 	return pass;
 }
 
+#ifdef CRYPTOPP_REMOVED
 bool ValidateXMACC()
 {
 	typedef XMACC<MD5> XMACC_MD5;
@@ -558,6 +544,7 @@ bool ValidateXMACC()
 
 	return pass;
 }
+#endif
 
 bool ValidateTTMAC()
 {
